@@ -120,7 +120,7 @@ class PrimitiveTree(list):
             if token in pset.mapping:
                 primitive = pset.mapping[token]
 
-                if len(ret_types) != 0 and primitive.ret != type_:
+                if type_ is not None and not issubclass(primitive.ret, type_):
                     raise TypeError("Primitive {} return type {} does not "
                                     "match the expected one: {}."
                                     .format(primitive, primitive.ret, type_))
@@ -137,7 +137,7 @@ class PrimitiveTree(list):
                 if type_ is None:
                     type_ = type(token)
 
-                if type(token) != type_:
+                if not issubclass(type(token), type_):
                     raise TypeError("Terminal {} type {} does not "
                                     "match the expected one: {}."
                                     .format(token, type(token), type_))
@@ -224,12 +224,6 @@ class Ephemeral(Terminal):
     def __init__(self):
         Terminal.__init__(self, self.func(), symbolic=False, ret=self.ret)
 
-    def regen(self):
-        """Regenerate the ephemeral value.
-        """
-        self.value = self.func()
-        self.name = str(self.value)
-
     @staticmethod
     def func():
         """Return a random value used to define the ephemeral state.
@@ -277,10 +271,13 @@ class PrimitiveSetTyped(object):
     def _add(self, prim):
         def addType(dict_, ret_type):
             if not ret_type in dict_:
-                dict_[ret_type]
+                new_list = []
                 for type_, list_ in dict_.items():
                     if issubclass(type_, ret_type):
-                        dict_[ret_type].extend(list_)
+                        for item in list_:
+                            if not item in new_list:
+                                new_list.append(item)
+                dict_[ret_type] = new_list
 
         addType(self.primitives, prim.ret)
         addType(self.terminals, prim.ret)
@@ -487,7 +484,7 @@ def compileADF(expr, psets):
 ######################################
 # GP Program generation functions    #
 ######################################
-def genFull(pset, min_, max_, type_=__type__):
+def genFull(pset, min_, max_, type_=None):
     """Generate an expression where each leaf has a the same depth
     between *min* and *max*.
 
@@ -503,7 +500,7 @@ def genFull(pset, min_, max_, type_=__type__):
         return depth == height
     return generate(pset, min_, max_, condition, type_)
 
-def genGrow(pset, min_, max_, type_=__type__):
+def genGrow(pset, min_, max_, type_=None):
     """Generate an expression where each leaf might have a different depth
     between *min* and *max*.
 
@@ -522,7 +519,7 @@ def genGrow(pset, min_, max_, type_=__type__):
                (depth >= min_ and random.random() < pset.terminalRatio)
     return generate(pset, min_, max_, condition, type_)
 
-def genHalfAndHalf(pset, min_, max_, type_=__type__):
+def genHalfAndHalf(pset, min_, max_, type_=None):
     """Generate an expression with a PrimitiveSet *pset*.
     Half the time, the expression is generated with :func:`~deap.gp.genGrow`,
     the other half, the expression is generated with :func:`~deap.gp.genFull`.
@@ -537,7 +534,7 @@ def genHalfAndHalf(pset, min_, max_, type_=__type__):
     method = random.choice((genGrow, genFull))
     return method(pset, min_, max_, type_)
 
-def genRamped(pset, min_, max_, type_=__type__):
+def genRamped(pset, min_, max_, type_=None):
     """
     .. deprecated:: 1.0
         The function has been renamed. Use :func:`~deap.gp.genHalfAndHalf` instead.
@@ -546,7 +543,7 @@ def genRamped(pset, min_, max_, type_=__type__):
                   FutureWarning)
     return genHalfAndHalf(pset, min_, max_, type_)
 
-def generate(pset, min_, max_, condition, type_=__type__):
+def generate(pset, min_, max_, condition, type_=None):
     """Generate a Tree as a list of list. The tree is build
     from the root to the leaves, and it stop growing when the
     condition is fulfilled.
@@ -562,6 +559,8 @@ def generate(pset, min_, max_, condition, type_=__type__):
     :returns: A grown tree with leaves at possibly different depths
               dependending on the condition function.
     """
+    if type_ is None:
+        type_ = pset.ret
     expr = []
     height = random.randint(min_, max_)
     stack = [(0, type_)]
@@ -759,7 +758,7 @@ def mutEphemeral(individual, mode):
             ephemerals_idx = (random.choice(ephemerals_idx),)
 
         for i in ephemerals_idx:
-            individual[i].regen()
+            individual[i] = type(individual[i])()
 
     return individual,
 
